package com.example.bondoyap

import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.content.SharedPreferences
import android.os.Bundle
import androidx.activity.viewModels
import com.google.android.material.bottomnavigation.BottomNavigationView
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContentProviderCompat.requireContext
import androidx.fragment.app.viewModels
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import androidx.navigation.findNavController
import androidx.navigation.ui.AppBarConfiguration
import androidx.navigation.ui.setupActionBarWithNavController
import androidx.navigation.ui.setupWithNavController
import com.example.bondoyap.service.api.Constants.SHARED_PREFS_NAME
import com.example.bondoyap.databinding.ActivityMainBinding
import com.example.bondoyap.service.api.Constants
import com.example.bondoyap.service.jwt.JwtService
import com.example.bondoyap.ui.login.LoginActivity
import com.example.bondoyap.ui.transactions.TransactionsApplication
import com.example.bondoyap.ui.transactions.TransactionsBroadcastReceiver
import com.example.bondoyap.ui.transactions.TransactionsViewModel
import com.example.bondoyap.ui.transactions.TransactionsViewModelFactory

class MainActivity : AppCompatActivity() {

    private lateinit var binding: ActivityMainBinding
    private lateinit var sharedPreferences: SharedPreferences
    private lateinit var transactionsBroadcastReceiver: TransactionsBroadcastReceiver

    private val transactionsViewModel: TransactionsViewModel by viewModels {
        TransactionsViewModelFactory((applicationContext as TransactionsApplication).repository)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        sharedPreferences = getSharedPreferences(SHARED_PREFS_NAME, Context.MODE_PRIVATE)

        if (!isLoggedIn()) {
            val intent = Intent(this, LoginActivity::class.java)
            startActivity(intent)
            finish()
        }

        val serviceIntent = Intent(this, JwtService::class.java)
        this.startService(serviceIntent)

        binding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(binding.root)

        val navView: BottomNavigationView = binding.navView

        val navController = findNavController(R.id.nav_host_fragment_activity_main)

        val appBarConfiguration = AppBarConfiguration(setOf(
                R.id.navigation_settings, R.id.navigation_transactions, R.id.navigation_scanner,
                R.id.navigation_graph))

        setupActionBarWithNavController(navController, appBarConfiguration)

        navView.setupWithNavController(navController)

        transactionsBroadcastReceiver = TransactionsBroadcastReceiver(transactionsViewModel)
        val filter = IntentFilter(Constants.ACTION_RANDOMIZE_TRANSACTIONS)
        LocalBroadcastManager.getInstance(this).registerReceiver(transactionsBroadcastReceiver, filter)
    }

    override fun onSupportNavigateUp(): Boolean {
        val navController = findNavController(R.id.nav_host_fragment_activity_main)
        return navController.navigateUp() || super.onSupportNavigateUp()
    }
    private fun isLoggedIn(): Boolean {
        return sharedPreferences.getBoolean("isLoggedIn", false)
    }

    override fun onDestroy() {
        super.onDestroy()
        LocalBroadcastManager.getInstance(this).unregisterReceiver(transactionsBroadcastReceiver)
    }
}