package com.example.bondoyap.ui.login

import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import androidx.lifecycle.Observer
import androidx.lifecycle.ViewModelProvider
import androidx.annotation.StringRes
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.view.View
import android.view.inputmethod.EditorInfo
import android.widget.EditText
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import com.example.bondoyap.service.api.Constants.SHARED_PREFS_NAME
import com.example.bondoyap.MainActivity
import com.example.bondoyap.R
import com.example.bondoyap.databinding.ActivityLoginBinding

class LoginActivity : AppCompatActivity() {

    private lateinit var loginViewModel: LoginViewModel
    private lateinit var binding: ActivityLoginBinding
    private lateinit var sharedPreferences: SharedPreferences


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        sharedPreferences = getSharedPreferences(SHARED_PREFS_NAME, Context.MODE_PRIVATE)



        if (isLoggedIn()) {
            navigateToMainActivity()
            finish()
        } else {
            binding = ActivityLoginBinding.inflate(layoutInflater)
            setContentView(binding.root)

            val emailEditText = binding.email
            val passwordEditText = binding.password
            val loginButton = binding.login
            val loadingProgressBar = binding.loading

            val factory = LoginViewModelFactory(this)
            loginViewModel = ViewModelProvider(this, factory)[LoginViewModel::class.java]

            // skip login
            loginViewModel.login(
                "13521xxx@std.stei.itb.ac.id",
                "password_13521xxx"
            )
            if (isLoggedIn()) {
                navigateToMainActivity()
                finish()
            }

            loginViewModel.loginFormState.observe(this@LoginActivity, Observer {
                val loginState = it ?: return@Observer

                // disable login button unless both username / password is valid
                loginButton.isEnabled = loginState.isDataValid

                if (loginState.emailError != null) {
                    emailEditText.error = getString(loginState.emailError)
                }
                if (loginState.passwordError != null) {
                    passwordEditText.error = getString(loginState.passwordError)
                }
            })

            loginViewModel.loginResult.observe(this@LoginActivity, Observer {
                val loginResult = it ?: return@Observer

                loadingProgressBar.visibility = View.GONE
                if (loginResult.error != null) {
                    showLoginFailed(loginResult.error)
                }
                if (loginResult.success != null) {
                    updateUiWithUser(loginResult.success)
                    setResult(RESULT_OK)
                    finish()
                }
            })

            emailEditText.afterTextChanged {
                loginViewModel.loginDataChanged(
                    emailEditText.text.toString(),
                    passwordEditText.text.toString()
                )
            }

            passwordEditText.apply {
                afterTextChanged {
                    loginViewModel.loginDataChanged(
                        emailEditText.text.toString(),
                        passwordEditText.text.toString()
                    )
                }

                setOnEditorActionListener { _, actionId, _ ->
                    when (actionId) {
                        EditorInfo.IME_ACTION_DONE ->
                            loginViewModel.login(
                                emailEditText.text.toString(),
                                passwordEditText.text.toString()
                            )
                    }
                    false
                }

                loginButton.setOnClickListener {
                    loadingProgressBar.visibility = View.VISIBLE
                    loginViewModel.login(
                        emailEditText.text.toString(),
                        passwordEditText.text.toString()
                    )
                }
            }
        }
    }

    private fun updateUiWithUser(model: LoggedInUserView) {
        val welcome = getString(R.string.welcome) + "\n" + model.email
        // TODO : initiate successful logged in experience
        Toast.makeText(
            applicationContext,
            welcome,
            Toast.LENGTH_SHORT
        ).show()
        navigateToMainActivity()
    }

    private fun showLoginFailed(@StringRes errorString: Int) {
        Toast.makeText(applicationContext, errorString, Toast.LENGTH_SHORT).show()
    }

    private fun navigateToMainActivity() {
        val intent = Intent(this, MainActivity::class.java)
        startActivity(intent)
        finish()
    }

    private fun isLoggedIn(): Boolean {
        return sharedPreferences.getBoolean("isLoggedIn", false)
    }
}

/**
 * Extension function to simplify setting an afterTextChanged action to EditText components.
 */
fun EditText.afterTextChanged(afterTextChanged: (String) -> Unit) {
    this.addTextChangedListener(object : TextWatcher {
        override fun afterTextChanged(editable: Editable?) {
            afterTextChanged.invoke(editable.toString())
        }

        override fun beforeTextChanged(s: CharSequence, start: Int, count: Int, after: Int) {}

        override fun onTextChanged(s: CharSequence, start: Int, before: Int, count: Int) {}
    })
}