package com.example.bondoyap.ui.settings

import android.app.AlertDialog
import android.content.DialogInterface
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.lifecycle.ViewModelProvider
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import com.example.bondoyap.databinding.FragmentSettingsBinding
import com.example.bondoyap.service.api.Constants.ACTION_RANDOMIZE_TRANSACTIONS
import com.example.bondoyap.ui.login.LoginActivity
import com.example.bondoyap.ui.transactions.TransactionsApplication
import com.example.bondoyap.ui.transactions.TransactionsViewModel
import com.example.bondoyap.ui.transactions.TransactionsViewModelFactory
import android.Manifest

class SettingsFragment : Fragment() {

    private var _binding: FragmentSettingsBinding? = null
    private val binding get() = _binding!!
    private lateinit var settingsViewModel: SettingsViewModel

    private val transactionsViewModel: TransactionsViewModel by viewModels {
        TransactionsViewModelFactory((requireContext().applicationContext as TransactionsApplication).repository)
    }

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentSettingsBinding.inflate(inflater, container, false)

        if (ContextCompat.checkSelfPermission(requireContext(), Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(requireActivity(), arrayOf(Manifest.permission.WRITE_EXTERNAL_STORAGE), PERMISSION_REQUEST_CODE)
        }

        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        val factory = context?.let { SettingsViewModelFactory(it) }
        if (factory != null) {
            settingsViewModel = ViewModelProvider(this, factory)[SettingsViewModel::class.java]
        } else {
            throw IllegalStateException("Context is null. Cannot create SettingsViewModelFactory.")
        }
        val textView = binding.textSettings
        val logoutButton = binding.logoutButton
        val randomButton = binding.randomTransactions
        val saveButton = binding.saveTransactions
        val sendButton = binding.sendTransactions

        settingsViewModel.getUser()?.let { user ->
            val loggedInUserText = "Masuk dengan akun:\n ${user.email}"
            textView.text = loggedInUserText
        }
        val appContext = context?.applicationContext

        logoutButton.setOnClickListener {
            settingsViewModel.getUser()
            settingsViewModel.logout()
//            findNavController().navigate(R.id.navigation_login)

            val activity = requireActivity()
            val intent = Intent(activity, LoginActivity::class.java)
            Toast.makeText(appContext, "Logout Sukses!", Toast.LENGTH_SHORT).show()
            activity.startActivity(intent)
            activity.finish()
        }

        randomButton.setOnClickListener {
            Toast.makeText(appContext, "Membuat transaksi random ...", Toast.LENGTH_SHORT).show()

            val intent = Intent(ACTION_RANDOMIZE_TRANSACTIONS)
            intent.putExtra("message", "Randomize from setting!")
            LocalBroadcastManager.getInstance(requireContext()).sendBroadcast(intent)

            Log.d("BroadcastDebug", "Sending broadcast from SettingsFragment")
        }

        val exporter = TransactionsExporter(transactionsViewModel, requireContext())

        saveButton.setOnClickListener {
            if (ContextCompat.checkSelfPermission(requireContext(), Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
                Toast.makeText(appContext, "Allow storage permission untuk menyimpan transaksi ke file xls/xlsx", Toast.LENGTH_SHORT).show()
            } else {
                val formats = arrayOf("XLS", "XLSX")
                val builder = AlertDialog.Builder(requireContext())
                builder.setTitle("Pilih Format File")
                builder.setItems(formats) { dialog: DialogInterface, which: Int ->
                    when (which) {
                        0 -> {
                            Toast.makeText(appContext, "Menyimpan transaksi ke xls...", Toast.LENGTH_SHORT).show()
                            exporter.exportToXLS()
                            Toast.makeText(appContext, "Penyimpanan xls pada folder Documents berhasil...", Toast.LENGTH_SHORT).show()
                        }
                        1 -> {
                            Toast.makeText(appContext, "Menyimpan transaksi ke xlsx...", Toast.LENGTH_SHORT).show()
                            exporter.exportToXLSX()
                            Toast.makeText(appContext, "Penyimpanan xlsx pada folder Documents berhasil...", Toast.LENGTH_SHORT).show()
                        }
                    }
                    dialog.dismiss()
                }
                builder.create().show()
            }
        }

        sendButton.setOnClickListener {
            Toast.makeText(appContext, "Mengirimkan transaksi ...", Toast.LENGTH_SHORT).show()
            //todo
        }

    }

    companion object {
        private const val PERMISSION_REQUEST_CODE = 1001
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}