package com.example.bondoyap.ui.settings

import android.content.Context
import android.location.Address
import android.location.Geocoder
import android.os.Environment
import android.os.Handler
import android.os.Looper
import android.util.Log
import androidx.lifecycle.viewModelScope
import com.example.bondoyap.ui.transactions.TransactionsViewModel
import com.example.bondoyap.ui.transactions.data.Transactions
import io.github.evanrupert.excelkt.Sheet
import io.github.evanrupert.excelkt.workbook
import kotlinx.coroutines.launch
import org.apache.poi.ss.usermodel.FillPatternType
import org.apache.poi.ss.usermodel.IndexedColors
import java.io.File
import java.util.Locale

class TransactionsExporter(private val transactionsViewModel: TransactionsViewModel, val context: Context) {
    fun exportToXLS() {
        transactionsViewModel.viewModelScope.launch {
            val transactions = transactionsViewModel.getAllTransactionsList()
            Log.d("SaveDebug", "xls function")
            writeToExcel("transactions.xls", transactions)
        }
    }

    fun exportToXLSX() {
        transactionsViewModel.viewModelScope.launch {
            val transactions = transactionsViewModel.getAllTransactionsList()
            Log.d("SaveDebug", "xlsx function")
            writeToExcel("transactions.xlsx", transactions)
        }
    }

    private fun writeToExcel(fileName: String, transactions: List<Transactions>) {
        val downloadsDir = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOCUMENTS)
        val file = File(downloadsDir, fileName)

        workbook {
            sheet("Transactions") {
                transactionsHeader()

                for (transaction in transactions) {
                    row {
                        cell(transaction.tanggal)
                        cell(
                            if (transaction.isPemasukan) {
                                "Pemasukan"
                            } else {
                                "Pengeluaran"
                            }
                        )
                        cell(transaction.nominal)
                        cell(transaction.judul)
                        cell(if (transaction.longitude.isEmpty() || transaction.latitude.isEmpty()) {
                            "Unavailable"
                        } else {
                            val addresses: List<Address> =
                                Geocoder(context, Locale.getDefault()).getFromLocation(
                                    transaction.latitude.toDouble(),
                                    transaction.longitude.toDouble(),
                                    1
                                ) ?: emptyList()
                            if (addresses.isNotEmpty()) {
                                val locationName = addresses[0].getAddressLine(0)
                                locationName
                            } else {
                                "Unavailable"
                            }
                        })
                    }
                }
            }
        }.write(file.absolutePath)
    }


    private fun Sheet.transactionsHeader() {
        val headings = listOf("Tanggal", "Kategori Transaksi", "Nominal Transaksi", "Nama Transaksi", "Lokasi")

        val headingStyle = createCellStyle {
            setFont(createFont {
                fontName = "IMPACT"
                color = IndexedColors.BLACK.index
            })

            fillPattern = FillPatternType.SOLID_FOREGROUND
            fillForegroundColor = IndexedColors.YELLOW.index
        }

        row(headingStyle) {
            headings.forEach { cell(it) }
        }
    }
}