package com.example.bondoyap.ui.transactions

import android.R
import android.content.Context
import android.content.IntentFilter
import android.content.pm.PackageManager
import android.os.Build
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ArrayAdapter
import android.widget.Toast
import androidx.annotation.RequiresApi
import androidx.core.app.ActivityCompat
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import com.example.bondoyap.databinding.FragmentAddTransactionsBinding
import com.example.bondoyap.service.api.Constants.ACTION_RANDOMIZE_TRANSACTIONS
import com.example.bondoyap.ui.transactions.data.Transactions
import com.google.android.gms.location.FusedLocationProviderClient
import com.google.android.gms.location.LocationServices
import java.text.SimpleDateFormat
import java.util.Date
import java.util.Locale

class AddTransactionsFragment : Fragment() {

    private var _binding: FragmentAddTransactionsBinding? = null

    // This property is only valid between onCreateView and
    // onDestroyView.
    private val binding get() = _binding!!

    private val transactionsViewModel: TransactionsViewModel by viewModels {
        TransactionsViewModelFactory((requireContext().applicationContext as TransactionsApplication).repository)
    }

    private lateinit var fusedLocationProviderClient: FusedLocationProviderClient
    private lateinit var latitude: String
    private lateinit var longitude: String

    override fun onCreateView(
            inflater: LayoutInflater,
            container: ViewGroup?,
            savedInstanceState: Bundle?
    ): View {
        _binding = FragmentAddTransactionsBinding.inflate(inflater, container, false)

        val pemasukan = "Pemasukan"
        val pengeluaran = "Pengeluaran"

        val categories = arrayOf(pemasukan, pengeluaran)
        val adapter = ArrayAdapter(requireContext(), R.layout.simple_spinner_item, categories)

        adapter.setDropDownViewResource(R.layout.simple_spinner_dropdown_item)
        binding.spinnerKategori.adapter = adapter

        fusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(requireContext())

        if (
            ActivityCompat.checkSelfPermission(requireContext(), android.Manifest.permission.ACCESS_FINE_LOCATION)
            != PackageManager.PERMISSION_GRANTED
            && ActivityCompat.checkSelfPermission(requireContext(), android.Manifest.permission.ACCESS_COARSE_LOCATION)
            != PackageManager.PERMISSION_GRANTED
        ) {
            ActivityCompat.requestPermissions(requireActivity(),
                arrayOf(android.Manifest.permission.ACCESS_FINE_LOCATION), 100
            )
        }

        binding.buttonSimpan.setOnClickListener {
            val isPemasukan: Boolean = when (binding.spinnerKategori.selectedItem.toString()) {
                pemasukan -> true
                else -> false
            }

            val dateFormat = SimpleDateFormat("dd/MM/yyyy", Locale.getDefault())
            val currentDate = dateFormat.format(Date())

            val judul = if (binding.editTextJudul.text.toString().trim().isNotEmpty()) {
                binding.editTextJudul.text.toString()
            } else {
                "Untitled"
            }

            val nominal = if (binding.editTextNominal.text.toString().trim().isNotEmpty()) {
                binding.editTextNominal.text.toString().toDouble()
            } else {
                0.0
            }

                val location = fusedLocationProviderClient.lastLocation
            location.addOnSuccessListener {
                if(it != null) {
                    latitude = it.latitude.toString()
                    longitude = it.longitude.toString()

                    val transaction: Transactions = Transactions(
                        judul = judul,
                        nominal = nominal,
                        isPemasukan = isPemasukan,
                        tanggal = currentDate,
                        longitude = longitude,
                        latitude = latitude
                    )
                    transactionsViewModel.upsert(transaction)
                }
            }

            if(
                ActivityCompat.checkSelfPermission(requireContext(), android.Manifest.permission.ACCESS_FINE_LOCATION)
                != PackageManager.PERMISSION_GRANTED
                && ActivityCompat.checkSelfPermission(requireContext(), android.Manifest.permission.ACCESS_COARSE_LOCATION)
                != PackageManager.PERMISSION_GRANTED
            ) {
                val transaction: Transactions = Transactions(
                    judul = judul,
                    nominal = nominal,
                    isPemasukan = isPemasukan,
                    tanggal = currentDate,
                    longitude = "",
                    latitude = ""
                )
                transactionsViewModel.upsert(transaction)
            }

            Toast.makeText(requireContext(), "Transaksi berhasil disimpan", Toast.LENGTH_SHORT).show()

            binding.editTextJudul.text.clear()
            binding.editTextNominal.text.clear()
            binding.spinnerKategori.setSelection(0)
        }

        return binding.root
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}