package com.example.bondoyap.ui.transactions

import android.R
import android.app.AlertDialog
import android.content.pm.PackageManager
import android.location.Address
import android.location.Geocoder
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.text.SpannableStringBuilder
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ArrayAdapter
import android.widget.Toast
import androidx.core.app.ActivityCompat
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.navigation.fragment.findNavController
import com.example.bondoyap.databinding.FragmentEditTransactionsBinding
import com.example.bondoyap.ui.transactions.data.Transactions
import com.google.android.gms.location.FusedLocationProviderClient
import com.google.android.gms.location.LocationServices
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers.Main
import kotlinx.coroutines.launch
import java.util.Locale

class EditTransactionsFragment : Fragment() {

    private var _binding: FragmentEditTransactionsBinding? = null

    // This property is only valid between onCreateView and
    // onDestroyView.
    private val binding get() = _binding!!

    private val transactionsViewModel: TransactionsViewModel by viewModels {
        TransactionsViewModelFactory((requireContext().applicationContext as TransactionsApplication).repository)
    }

    private lateinit var fusedLocationProviderClient: FusedLocationProviderClient

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentEditTransactionsBinding.inflate(inflater, container, false)

        var tanggal: String = ""
        var latitude = ""
        var longitude = ""

        val pemasukan = "Pemasukan"
        val pengeluaran = "Pengeluaran"

        val categories = arrayOf(pemasukan, pengeluaran)
        val adapter = ArrayAdapter(requireContext(), R.layout.simple_spinner_item, categories)

        adapter.setDropDownViewResource(R.layout.simple_spinner_dropdown_item)
        binding.spinnerKategori.adapter = adapter

        binding.spinnerKategori.isEnabled = false
        binding.editTextLokasi.isEnabled = false

        val transactionId: Int = arguments?.getInt("transaction_id") ?: -1

        CoroutineScope(Main).launch {

            val originalTransaction: Transactions = transactionsViewModel.get(transactionId)

            binding.editTextJudul.text = SpannableStringBuilder(originalTransaction.judul)
            binding.editTextNominal.text = SpannableStringBuilder(originalTransaction.nominal.toBigDecimal().toString())

            if (originalTransaction.isPemasukan) {
                binding.spinnerKategori.setSelection(categories.indexOf(pemasukan))
            } else {
                binding.spinnerKategori.setSelection(categories.indexOf(pengeluaran))
            }

            tanggal = originalTransaction.tanggal
            longitude = originalTransaction.longitude
            latitude = originalTransaction.latitude

            if (originalTransaction.longitude.isEmpty() || originalTransaction.latitude.isEmpty()) {
                binding.editTextLokasi.text = SpannableStringBuilder("Unavailable")
            } else {
                val addresses: List<Address> =
                    Geocoder(requireContext(), Locale.getDefault()).getFromLocation(
                        originalTransaction.latitude.toDouble(),
                        originalTransaction.longitude.toDouble(),
                        1
                    ) ?: emptyList()
                if (addresses.isNotEmpty()) {
                    val locationName = addresses[0].getAddressLine(0)
                    Handler(Looper.getMainLooper()).post {
                        binding.editTextLokasi.text = SpannableStringBuilder(locationName)
                    }
                }
            }
        }

        fusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(requireContext())

        if (
            ActivityCompat.checkSelfPermission(requireContext(), android.Manifest.permission.ACCESS_FINE_LOCATION)
            != PackageManager.PERMISSION_GRANTED
            && ActivityCompat.checkSelfPermission(requireContext(), android.Manifest.permission.ACCESS_COARSE_LOCATION)
            != PackageManager.PERMISSION_GRANTED
        ) {
            ActivityCompat.requestPermissions(requireActivity(),
                arrayOf(android.Manifest.permission.ACCESS_FINE_LOCATION), 100
            )
        }

        binding.checkboxUpdateLokasi.setOnCheckedChangeListener { _, isChecked ->
            if (isChecked) {
                if (
                    ActivityCompat.checkSelfPermission(requireContext(), android.Manifest.permission.ACCESS_FINE_LOCATION)
                    != PackageManager.PERMISSION_GRANTED
                    && ActivityCompat.checkSelfPermission(requireContext(), android.Manifest.permission.ACCESS_COARSE_LOCATION)
                    != PackageManager.PERMISSION_GRANTED
                ) {
                    ActivityCompat.requestPermissions(requireActivity(),
                        arrayOf(android.Manifest.permission.ACCESS_FINE_LOCATION), 100
                    )
                    binding.checkboxUpdateLokasi.isChecked = false
                }
            }
        }

        binding.buttonUpdate.setOnClickListener {
            val isPemasukan: Boolean = when (binding.spinnerKategori.selectedItem.toString()) {
                pemasukan -> true
                else -> false
            }

            val judul = if (binding.editTextJudul.text.toString().trim().isNotEmpty()) {
                binding.editTextJudul.text.toString()
            } else {
                "Untitled"
            }

            val nominal = if (binding.editTextNominal.text.toString().trim().isNotEmpty()) {
                binding.editTextNominal.text.toString().toDouble()
            } else {
                0.0
            }

            val transaction: Transactions = Transactions(
                judul = judul,
                nominal = nominal,
                isPemasukan = isPemasukan,
                tanggal = tanggal,
                longitude = longitude,
                latitude = latitude,
                id = transactionId
            )

            if (binding.checkboxUpdateLokasi.isChecked) {
                val location = fusedLocationProviderClient.lastLocation
                location.addOnSuccessListener {
                    if (it != null) {
                        latitude = it.latitude.toString()
                        longitude = it.longitude.toString()

                        transaction.latitude = latitude
                        transaction.longitude = longitude
                    }
                }
            }

            showConfirmationDialog("Update", "Apakah Anda yakin ingin memperbarui transaksi ini?") {
                transactionsViewModel.upsert(transaction)

                binding.editTextJudul.text = SpannableStringBuilder(transaction.judul)
                binding.editTextNominal.text =
                    SpannableStringBuilder(transaction.nominal.toBigDecimal().toString())

                if (transaction.longitude.isEmpty() || transaction.latitude.isEmpty()) {
                    binding.editTextLokasi.text = SpannableStringBuilder("Unavailable")
                } else {
                    val addresses: List<Address> =
                        Geocoder(requireContext(), Locale.getDefault()).getFromLocation(
                            transaction.latitude.toDouble(),
                            transaction.longitude.toDouble(),
                            1
                        ) ?: emptyList()
                    if (addresses.isNotEmpty()) {
                        val locationName = addresses[0].getAddressLine(0)
                        Handler(Looper.getMainLooper()).post {
                            binding.editTextLokasi.text = SpannableStringBuilder(locationName)
                        }
                    }
                }

                if (transaction.isPemasukan) {
                    binding.spinnerKategori.setSelection(categories.indexOf(pemasukan))
                } else {
                    binding.spinnerKategori.setSelection(categories.indexOf(pengeluaran))
                }
                Toast.makeText(requireContext(), "Transaksi berhasil diperbarui", Toast.LENGTH_SHORT).show()
            }


        }

        binding.buttonHapus.setOnClickListener {
            val transaction: Transactions = Transactions(
                judul = "",
                nominal = 0.0,
                isPemasukan = false,
                tanggal = "",
                id = transactionId
            )

            showConfirmationDialog("Hapus", "Apakah Anda yakin ingin menghapus transaksi ini?") {
                transactionsViewModel.delete(transaction)
                findNavController().navigate(com.example.bondoyap.R.id.navigation_transactions)
                Toast.makeText(requireContext(), "Transaksi berhasil dihapus", Toast.LENGTH_SHORT).show()
            }
        }

        return binding.root
    }

    private fun showConfirmationDialog(title: String, message: String, action: () -> Unit) {
        val builder = AlertDialog.Builder(requireContext())
        builder.setTitle(title)
        builder.setMessage(message)
        builder.setPositiveButton("Ya") { _, _ ->
            action.invoke()
        }
        builder.setNegativeButton("Tidak") { dialog, _ ->
            dialog.dismiss()
        }
        val dialog = builder.create()
        dialog.show()
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}