package com.example.bondoyap.ui.transactions

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.util.Log
import android.widget.Toast
import androidx.core.app.ActivityCompat
import com.example.bondoyap.ui.transactions.data.Transactions
import com.google.android.gms.location.FusedLocationProviderClient
import com.google.android.gms.location.LocationServices
import java.text.SimpleDateFormat
import java.util.Date
import java.util.Locale
import kotlin.random.Random

class TransactionsBroadcastReceiver(private val transactionsViewModel: TransactionsViewModel) : BroadcastReceiver() {
    private lateinit var fusedLocationProviderClient: FusedLocationProviderClient
    private lateinit var latitude: String
    private lateinit var longitude: String
    override fun onReceive(context: Context?, intent: Intent?) {
        Log.d("BroadcastDebug", "Broadcast received in AddTransactionsFragment")

        context ?: return

        val randomJudul = "Random Judul ${Random.nextInt(1000)}"
        val randomNominal = Random.nextDouble(1000000000000000.0)
        val randomIsPemasukan = Random.nextBoolean()
        val dateFormat = SimpleDateFormat("dd/MM/yyyy", Locale.getDefault())
        val currentDate = dateFormat.format(Date())

        fusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(context)
        val location = fusedLocationProviderClient.lastLocation
        location.addOnSuccessListener {
            if(it != null) {
                latitude = it.latitude.toString()
                longitude = it.longitude.toString()

                val transaction: Transactions = Transactions(
                    judul = randomJudul,
                    nominal = randomNominal,
                    isPemasukan = randomIsPemasukan,
                    tanggal = currentDate,
                    longitude = longitude,
                    latitude = latitude
                )
                transactionsViewModel.upsert(transaction)
            }
        }

        if(
            ActivityCompat.checkSelfPermission(context, android.Manifest.permission.ACCESS_FINE_LOCATION)
            != PackageManager.PERMISSION_GRANTED
            && ActivityCompat.checkSelfPermission(context, android.Manifest.permission.ACCESS_COARSE_LOCATION)
            != PackageManager.PERMISSION_GRANTED
        ) {
            val transaction: Transactions = Transactions(
                judul = randomJudul,
                nominal = randomNominal,
                isPemasukan = randomIsPemasukan,
                tanggal = currentDate,
                longitude = "",
                latitude = ""
            )
            transactionsViewModel.upsert(transaction)
            Toast.makeText(context.applicationContext,
                "Izinkan location permission untuk membuat transaksi random dengan lokasi",
                Toast.LENGTH_SHORT).show()
        }

        Toast.makeText(context.applicationContext, "Transaksi random telah dibuat", Toast.LENGTH_SHORT).show()
    }
}