package com.example.bondoyap.ui.transactions

import androidx.lifecycle.LiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.asLiveData
import androidx.lifecycle.viewModelScope
import com.example.bondoyap.ui.transactions.data.Transactions
import com.example.bondoyap.ui.transactions.data.TransactionsRepository
import kotlinx.coroutines.Deferred
import kotlinx.coroutines.async
import kotlinx.coroutines.launch

class TransactionsViewModel(
    private val repository: TransactionsRepository
): ViewModel() {
    val allTransactions: LiveData<List<Transactions>> = repository.allTransactions.asLiveData()

    fun upsert(transactions: Transactions) = viewModelScope.launch {
        repository.upsert(transactions)
    }
    fun delete(transactions: Transactions) = viewModelScope.launch {
        repository.delete(transactions)
    }
    suspend fun get(transactionId: Int?): Transactions {
        val deferred: Deferred<Transactions> = viewModelScope.async {
            repository.get(transactionId)
        }
        return deferred.await()
    }

    suspend fun getAllTransactionsList(): List<Transactions> {
        return repository.getAllTransactionsList()
    }
}

class TransactionsViewModelFactory(private val repository: TransactionsRepository): ViewModelProvider.Factory {
    override fun<T: ViewModel> create(modelClass: Class<T>): T {
        if (modelClass.isAssignableFrom(TransactionsViewModel::class.java)) {
            @Suppress("UNCHECKED_CAST")
            return TransactionsViewModel(repository) as T
        }
        throw IllegalArgumentException("Unknown ViewModel class")
    }
}