package com.example.bondoyap.ui.transactions.data

import android.content.Context
import androidx.room.Database
import androidx.room.Room
import androidx.room.RoomDatabase
import androidx.room.migration.Migration
import androidx.sqlite.db.SupportSQLiteDatabase

@Database(
    entities = [Transactions::class],
    version = 3,
    exportSchema = false
)
public abstract class TransactionsRoomDatabase: RoomDatabase() {

    abstract fun transactionsDao(): TransactionsDao

    companion object {
        @Volatile
        private var INSTANCE: TransactionsRoomDatabase? = null

        fun getDatabase(context: Context): TransactionsRoomDatabase {
            return INSTANCE ?: synchronized(this) {
                val instance = Room.databaseBuilder(
                    context.applicationContext,
                    TransactionsRoomDatabase::class.java,
                    "transactions_database"
                ).addMigrations(MIGRATION_2_3).
                build()
                INSTANCE = instance
                instance
            }
        }
        private val MIGRATION_1_2: Migration = object : Migration(1, 2) {
            override fun migrate(db: SupportSQLiteDatabase) {
                db.execSQL("CREATE TABLE IF NOT EXISTS `transactions_new` " +
                        "(`id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL, " +
                        "`judul` TEXT NOT NULL, " +
                        "`nominal` REAL NOT NULL, " +
                        "`is_pemasukan` INTEGER NOT NULL, " +
                        "`tanggal` TEXT NOT NULL, " +
                        "`lokasi` TEXT NOT NULL)")

                db.execSQL("INSERT INTO transactions_new (id, judul, nominal, is_pemasukan) " +
                        "SELECT id, judul, nominal, is_pemasukan FROM transactions")

                db.execSQL("DROP TABLE transactions")

                db.execSQL("ALTER TABLE transactions_new RENAME TO transactions")
            }
        }
        private val MIGRATION_2_3: Migration = object : Migration(2, 3) {
            override fun migrate(db: SupportSQLiteDatabase) {
                db.execSQL("CREATE TABLE IF NOT EXISTS `transactions_new` " +
                        "(`id` INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL, " +
                        "`judul` TEXT NOT NULL, " +
                        "`nominal` REAL NOT NULL, " +
                        "`is_pemasukan` INTEGER NOT NULL, " +
                        "`tanggal` TEXT NOT NULL, " +
                        "`longitude` TEXT NOT NULL, " +
                        "`latitude` TEXT NOT NULL)")

                db.execSQL("DROP TABLE transactions")

                db.execSQL("ALTER TABLE transactions_new RENAME TO transactions")
            }
        }
    }
}